/* Written by Paul Higgins June, 2016
 from the RGB2PCT.PY wriiten by  Frank Warmerdam
#******************************************************************************
#  Copyright (c) 2001, Frank Warmerdam
# 
#  Permission is hereby granted, free of charge, to any person obtaining a
#  copy of this software and associated documentation files (the "Software"),
#  to deal in the Software without restriction, including without limitation
#  the rights to use, copy, modify, merge, publish, distribute, sublicense,
#  and/or sell copies of the Software, and to permit persons to whom the
#  Software is furnished to do so, subject to the following conditions:
# 
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software.
# 
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
#  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
#  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.
#******************************************************************************
*/

#include <stdint.h>

#include <vector>

#include "common.h"


#if defined(_WIN32)
#define _CDECL __cdecl
#else
#define _CDECL
#endif

using namespace dangdal;      

// print('Usage: rgb2pct.py [-n colors | -pct palette_file] [-of format] source_file dest_file')

void usage(const std::string &cmdname) {
	printf("Usage:\n");
	printf("    %s -in <in.tif> -out <out.tif>\n", cmdname.c_str());
	printf("\nConverts the in.tif RGB file to out.tif with one band with 127 colors.\n");
	printf("Currently only 8-bit data is supported.\n");
	exit(1);
}       

int _CDECL main(int argc, char **argv)      
{
	const std::string cmdname = argv[0];    
	if(argc == 1) usage(cmdname);
	std::vector<std::string> arg_list = argv_to_list(argc, argv);

	std::string dst_fn;
   std::string src_fn;
   std::string tif_fn;
   
	// std::vector<GDALDatasetH> src_ds;
	std::string output_format = "GTiff";         
   

	GDALAllRegister();
   

	size_t argp = 1;
	while(argp < arg_list.size()) {
		const std::string &arg = arg_list[argp++];
		// FIXME - check duplicate values
		if(arg[0] == '-') {
			if(arg == "-out") 
         {
				if(argp == arg_list.size()) usage(cmdname); 
				dst_fn = arg_list[argp++];
			} 
         else if(arg == "-in") 
         { 
				if(argp == arg_list.size()) usage(cmdname);
				src_fn = arg_list[argp++];
			}
         else 
         {
				usage(cmdname);
			}
		} 
      else 
      {
			usage(cmdname);
		}
	}

	if(dst_fn.empty()) usage(cmdname);
   if(src_fn.empty()) usage(cmdname);


	//////// open source //////// 
   GDALDatasetH src_ds = GDALOpen(src_fn.c_str(), GA_ReadOnly);
	if(!src_ds) fatal_error("open failed for src_fn");
   
   // it must have 3 rasters
   if(GDALGetRasterCount(src_ds) < 3) fatal_error("src_ds need 3 rasters for input red, green and blue.");
   
   // Ensure we recognise the driver.
   GDALDriverH dst_driver = GDALGetDriverByName(output_format.c_str());
	if(!dst_driver) fatal_error("unrecognized output format (%s)", output_format.c_str());
   
   // Generate palette
   int color_count = 127;
   GDALColorTableH  ct;
   ct = GDALCreateColorTable((GDALPaletteInterp)1);
   if( ct == NULL )
        fatal_error("Color Table is Null.");
   
   int err =GDALComputeMedianCutPCT(GDALGetRasterBand(src_ds, 1),
                                    GDALGetRasterBand(src_ds, 2),
                                    GDALGetRasterBand(src_ds, 3),
                                    NULL,
                                    color_count, 
                                    ct,
                                    NULL, 
                                    NULL);
   tif_fn = dst_fn;
   
   GDALDatasetH tif_ds = GDALCreate(dst_driver, tif_fn.c_str(), GDALGetRasterXSize(src_ds), GDALGetRasterYSize(src_ds), 1 , (GDALDataType)1, NULL);
   
   GDALSetRasterColorTable(GDALGetRasterBand(tif_ds,1), ct );
   
   // ----------------------------------------------------------------------------
   //  We should copy projection information and so forth at this point.
   
   // PY tif_ds.SetProjection( src_ds.GetProjection() )    
   GDALSetProjection(tif_ds,  GDALGetProjectionRef(src_ds));
   
   /* PY
   tif_ds.SetGeoTransform( src_ds.GetGeoTransform() )   
   if src_ds.GetGCPCount() > 0:
    tif_ds.SetGCPs( src_ds.GetGCPs(), src_ds.GetGCPProjection() )
   */
   GDALSetGeoTransform(tif_ds, (double *)GDALGetGCPProjection(src_ds));
   if (GDALGetGCPCount(src_ds) > 0) {
    GDALSetGCPs(tif_ds, GDALGetGCPCount(src_ds), GDALGetGCPs(src_ds), GDALGetGCPProjection(src_ds));
   }
   
   // ----------------------------------------------------------------------------
   // Actually transfer and dither the data.

   err = GDALDitherRGB2PCT( GDALGetRasterBand(src_ds, 1),
                             GDALGetRasterBand(src_ds, 2),
                             GDALGetRasterBand(src_ds, 3),
                             GDALGetRasterBand(tif_ds, 1),
                             ct,
                             NULL,
                             NULL );
                             

   GDALClose(tif_ds);                             
   tif_ds = NULL;                             
   
   GDALClose(src_ds);
	// GDALClose(dst_ds);

	GDALTermProgress(1, NULL, NULL);

	return 0;
}